; Barrires passages  niveau  16F690  - JLF   - .ASM   sur PIC 16F690 de MICROCHIP
; Version V1.1                                                         Le 19/05/2021
; Tabulation pour Notepad++
;
; Version V1.1 du 19/05/2021 = Mise en service immdiate des clignotants (Suppression de la tempo de 500 ms).
;
; Ferme les barrires d'un passage  niveau en commandant la position de deux servomoteurs.
; Utilisable sur une maquette ferroviaire de train lectrique, au passage des locomotives.
; Fonctionne avec des servos de dbattement 90 ou 180.
;
; Montage : PIC 16F690 avec horloge interne  8 Mhz.
; Site    : http://www.la-tour.info/uts/uts_index.html
;
; http://www.strmtg.developpement-durable.gouv.fr/IMG/pdf/Guide_surete_PN_SAL_CFT_V1-0_signe.pdf
;   Clignotement = 7 secondes.                 Fermeture des barrires entre 6  10 secondes.
;   Passage du train entre 8 et 27 secondes.   Ouverture des barrires entre 2 et 8 secondes.
;
; Utilise deux sous programmes mathmatiques :
;   Unsigned division 24 bit / 16 bit -> 24 bit - http://www.piclist.com/techref/microchip/math/div/24by16.htm
;   Unsigned multiplication 16 bits * 16 bits -> 32 bits http://www.piclist.com/techref/microchip/math/mul/16x16umalin.htm
;
; Les feux clignotants :
;  - sallument ds le dclenchement de larrive du train.
;  - steignent au dbut de louverture des barrires.
; Les barrires :
;  - commencent leur fermeture de 0  5 sec aprs le dclenchement de larrive du train.
;  - se ferment et souvrent en 0,5  10 sec.
; La sonnerie :
;  - fonctionne ds le dclenchement de larrive du train jusqu' la fermeture complte des barrires.
;
; La tension d'alimentation des servo moteurs de 5 volts, peut tre rduite pour favoriser un dplacement plus fluide.
; Ne pas utiliser des servo 360, qui sont des servos  rotation continue.
;
; Horloge interne = 8 Mhz => cycle opration = 2 Mhz = 0,5 sec par instruction.
; Utilisation du Timer1 de 0  65535 * 0,5 s.
; Toutes les 20 msec, calcul et envoi de la consigne du servo.
; Temps mort = 40000 - Consigne_1 - Consigne_2 = 15 ms  19 ms.
;
; +5V ---+----------------+------------- - - -
;        <  22 Kohms      < 22 Kohms
;        >                >
;        +----> POS_A1    +----> POS_A2
;        <      Position  Position
;        >                >
; 0V  ---+----------------+------------- - - -
;
; L'utilisation des pattes du PIC 16F690
; Potentiomtres de 22 KOhms entre la masse et le + 5 volts
; RA0 - CAN0  = POT_DO, Temporisation pour l'ouverture des barrires, en absence de signal de train.
; RA1 - CAN1  = POT_DB, Temporisation pour la fermeture des barrires, aprs le clignotement.
; RA2 - CAN2  = POT_VF, Vitesse de fermeture, 0,5  10 secondes.
; RA3 - IN  = Entre de dtection du train = Prsence train = 0 volt.
; RA4 - IN  = Inversion de la course du Servo_B, inversion = cavalier en place.
; RA5 - IN  = Inversion de la course du Servo_A, inversion = cavalier en place.
;---------------
; RB4 - CAN10 = POT_A2, Servo_A, position ferm.
; RB5 - CAN11 = POT_A1, Servo_A, position ouvert.
; RB6 - IN  = Type de servo 90 ou 180.
; RB7 - IN  = Type de clignotement, 40% ou 50%.
;---------------
; RC0 - CAN4  = POT_VO, Vitesse d'ouverture, 0,5  10 secondes.
; RC1 - CAN5  = POT_B2, Servo_B, position ferm.
; RC2 - CAN6  = POT_B1, Servo_B, position ouvert.
; RC3 - OUT = Sortie vers SERVO B.
; RC4 - OUT = Sortie vers SERVO A.
; RC5 - OUT = Sortie activation sonnerie.
; RC6 - OUT = Sortie vers LED_A.
; RC7 - OUT = Sortie vers LED_B.
;
; ---------------------------------------------------------------------------------------------------------------------------------
;
; ETAT           : 0000111112222222222222233333333333333333333333333333333333333333333333333333333333333333333333333000000000000000
;
; TRAIN          : ----__________-______________________________________________---_______---____________--------------------------
;
; LED_1 et LED_2 : ----_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_---------------
;
; SON_1          : ----___________________-----------------------------------------------------------------------------------------
;
; SERVO          : ---------FFFFFFFFFFFFFF__________________________________________________________________________000000000------
;
;************************************************************************************************
; SERVO 90  : 1,0 msec  2,0 msec 
;       = (A + B*X)/C
; POTA1 = (26000  + 12*X)/13
;    0    26000   2000   1,000   ms
;  511    32132   2472   1,236   ms
; 1023    38276   2944   1,472   ms
; 
; POTA2  (39720  + 12*X)/13
;    0    39720   3055   1,528   ms
;  511    45852   3527   1,764   ms
; 1023    51996   4000   2,000   ms
;  
;------------------------------------
; SERVO 180  : 0,5 msec  2,5 msec
;       = (A + B*X)/C
; POTA1 = (13000  + 25*X)/13
;    0    13000   1000   0,500   ms
;  511    25775   1983   0,991   ms
; 1023    38575   2967   1,484   ms
;
; POTA2  (39720  + 25*X)/13
;    0    39720   3055   1,528   ms
;  511    52495   4038   2,019   ms
; 1023    65295   5023   2,511   ms
;
;
	LIST		P=PIC16F690		; Dfinition de processeur
	#include	<p16F690.inc>	; Dfinitions des variables

;	Code non protg,  oscill interne sans sortie, pas de dtection tension basse, pas d'utilisation de la patte MCLR, dmarrage lent, Watchdog hors service
	__config (_INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_OFF & _BOR_OFF & _IESO_OFF & _FCMEN_OFF)

	ERRORLEVEL  -302

	CBLOCK			0x020	; Dbut de la zone variables en BANK0 - 80 octets de 0x020  0x06F.
	phase : 		1		; Suivant le traitement de l'impulsion de sortie en cours = 0, 1, 2.
	etat1 :			1		; Champ de bits d'tat, voir #define.
	etat2 :			1		; Champ de bits d'tat, voir #define.

	cpt_etat :		1		; Compteur de squence de fonctionnement = 0, 1, 2, 3.
	cpt_led :		1		; Compteur pour le clignotement des leds.
	cpt_A_L :		1		; Pas unitaire d'incrmentation/dcrmentation servo A.
	cpt_A_H :		1		;   //
	cpt_B_L :		1		; Pas d'incrmentation/dcrmentation servo B.
	cpt_B_H :		1		;
	cp2_A_L :		1		; Pas unitaire d'incrmentation/dcrmentation servo A.
	cp2_A_H :		1		;   //
	cp2_B_L :		1		; Pas d'incrmentation/dcrmentation servo B.
	cp2_B_H :		1		;
	cpt_DB :		1		; Compteur pour la temporisation  la fermeture des barrires.
	cpt_DO :		1		; Compteur pour la temporisation  la fin de dtection du train.

	consigne_A_L :	1		; 130000  65505. Consigne servo A, avant la division par 13.
	consigne_A_H :	1		;   //
	consigne_B_L :	1		; 130000  65505. Consigne servo B, avant la division par 13.
	consigne_B_H :	1		;   //
	cons_A_L13 :	1		; 1000  2574 ou de 3465  5039. Consigne servo A, aprs la division par 13.
	cons_A_H13 :	1		;    //
	cons_B_L13 :	1		; 1000  2574 ou de 3465  5039. Consigne servo B, aprs la division par 13.
	cons_B_H13 :	1		;    //
	tpmort_L13 :	1		; 30000  38000. Temps mort de 15  19 msec, pour avoir des impulsions toutes les 20 msec.
	tpmort_H13 :	1		;    //

	POT_A1_L :		1		; 13000  33460 = Position 1 Servo A sur 16 bits, rfrence pour Timer1, avant la division par 13.
	POT_A1_H :		1		;
	POT_A2_L :		1		; 45045  65505 = Position 2 Servo A sur 16 bits, rfrence pour Timer1, avant la division par 13.
	POT_A2_H :		1		;
	POT_B1_L :		1		; 13000  33460 = Position 1 Servo B sur 16 bits, rfrence pour Timer1, avant la division par 13.
	POT_B1_H :		1		;
	POT_B2_L :		1		; 45045  65505 = Position 2 Servo B sur 16 bits, rfrence pour Timer1, avant la division par 13.
	POT_B2_H :		1		;
	POT_VO_L :		1		; 25  537 (Tempo) = 0,5  10 secondes = Dure d'ouverture des barrires.
	POT_VO_H :		1		;
	POT_VF_L :		1		; 25  537 (Tempo) = 0,4  10 secondes = Dure de fermeture des barrires.
	POT_VF_H :		1		;
	POT_DB :		1		; 1  255 (*20msec) = 0,1  5 secondes = Dlai entre le dbut du clignotement et la fermeture des barrires.
	POT_DO :		1		; 1  255 (*20msec) = 0,1  5 secondes = Dlai avant l'ouverture des barrires aprs le dernier wagon.

	temp1 :			1		; Variables temporaires.
	avirertemp1_L :		1
	avirertemp1_H : 		1

	aargb2 :		1		; call fxd2416u
	aargb1 :		1		;   Division d'entiers non signs : 24 bit / 16 bit ==> 24 bit
	aargb0 :		1		;   aargb / bargb ==> aargb
	bargb1 :		1		;   24 bit aargb0:aargb1:aargb2
	bargb0 :		1		;   16 bit bargb0:bargb1
	loopcount :		1		;   Registres temporaires pour la division.
	remb0 :			1		;     //
	remb1 :			1		;     //

	a1 :			1		; call Multiply16x16bit
	a2 :			1		; Multiplication d'entiers non signs : 16 bits * 16 bits ==> 32 bits.
	b1 :			1		; a2:a1 * b2:b1 ==> res4:res3:res2:res1
	b2 :			1		; 16 bit a2:a1
	res1 :			1		; 16 bit b2:b1
	res2 :			1		; 32 bit res4:res3:res2:res1
	res3 :			1		;
	res4 :			1		;
	ENDC

	CBLOCK			0x070	; Dbut de la zone des variables commune  toutes les BANK - 15 octets de 0x070  0x07F.
	w_temp :		1		; Donnes pour les sauvegardes dans la routine d'interruption.
	status_temp :	1		;
	temp1_L :		1		; Variables temporaires inter BANK.
	temp1_H : 		1		;   //
	ENDC

	CBLOCK			0x0A0	; Dbut de la zone des variables en BANK1 - 80 octets de 0x0A0  0x0EF.
	CAN5_H :		8		; Pour le calcul de la moyenne des 16 dernires mesures.
	CAN5_L :		8		;
	CAN6_H :		8		;
	CAN6_L :		8		;
	CAN10_H :		8		;
	CAN10_L :		8		;
	CAN11_H :		8		;
	CAN11_L :		8		;
	ENDC					; Fin de la zone

	CBLOCK			0x120	; Dbut de la zone des variables en BANK2 - 80 octets de 0x120  0x16F.
	CAN5_H2 :		8		; Pour le calcul de la moyenne des 16 dernires mesures.
	CAN5_L2 :		8		;
	CAN6_H2 :		8		;
	CAN6_L2 :		8		;
	CAN10_H2 :		8		;
	CAN10_L2 :		8		;
	CAN11_H2 :		8		;
	CAN11_L2 :		8		;
	ENDC					; Fin de la zone

;*************************************************************************************
; Dfinition des variables.

BANK0	macro			; Passer en banque0.
		bcf	STATUS,RP0
		bcf	STATUS,RP1
		endm
BANK1	macro			; Passer en banque1.
		bsf	STATUS,RP0
		bcf	STATUS,RP1
		endm
BANK2	macro			; Passer en banque2.
		bcf	STATUS,RP0
		bsf	STATUS,RP1
		endm

; Dfinition des bits de la variable 'etat1'
#define	bit_etat_OV		etat1,0		; Etat immdiat.  0 = Ouverture en cours, 1 = fermeture en cours.
#define	bit_etat_OVT	etat1,1		; Etat temporis. 0 = Ouverture en cours, 1 = fermeture en cours.
#define	bit_etat_OVA	etat1,2		; 0 = Ouverture en cours, 1 = Fermeture en cours pour le servo A.
#define	bit_etat_OVB	etat1,3		; 0 = Ouverture en cours, 1 = Fermeture en cours pour le servo B.
#define	bit_etat_cal	etat1,4		; 1 = Demande de lecture CAN et des calculs des variables.
#define	bit_etat_SON	etat1,5		; 1 = Active la sonnerie.
#define	bit_etat_LED	etat1,6		; 1 = active le clignotement des LED.
#define	bit_etat_BRF	etat1,7		; 1 = Les barrires sont compltement fermes.
#define	bit_etat_BRO	etat2,0		; 1 = Les barrires sont compltement ouvertes.
#define	bit_etat_TSV	etat2,1		; 0 = servo 90,  1 = servo 180.
#define	bit_etat_CLI	etat2,2		; 0 = cycle 50% , 1 = cycle 40%.
#define	bit_etat_CL2	etat2,3		; 0 = led A , 1 = led B en cours.
#define	WRbeCL2			b'00001000'	; 0 = led A , 1 = led B en cours.
#define	PRbeCL2			etat2		; 0 = led A , 1 = led B en cours.

; Dfinition des ports utiliss
#define	RAtrain			PORTA,3		; Entre de dtection du train
#define	RAinversA		PORTA,5		; Entre inversion de la course du Servo_A
#define	RAinversB		PORTA,4		; Entre inversion de la course du Servo_B
#define	RCservoA		PORTC,4		; Sortie servo A.
#define	RCservoB		PORTC,3		; Sortie servo B.
#define	RCledA			PORTC,6		; Sortie led A.
#define	RCledB			PORTC,7		; Sortie led B.
#define	RCsonnerie		PORTC,5		; Sortie sonnerie.
#define	RBtypesrv		PORTB,6		; Entre type de servo 90 ou 180.
#define	RBtypecli		PORTB,7		; Type de clignotement, 50% ou 40%.

;*************************************************************************************
; Dmarrage du programme  la mise sous tension.

		org			0x2100 	  			 ; Initialize EEPROM Data
		dt 			"Barrieres de passage a niveau a servo V1.1 "	; Texte visible dans le '.hex'.
		dt			"Le 19/05/2021 "
		dt			"http://www.la-tour.info/uts/uts_index.html"
		org	 		0x000				; Adresse de dpart aprs raz.
		goto		init
		org 		0x004				; Adresse de dpart aprs interruption.
		goto		interrup
;*************************************************************************************
		dt			"__BARRIERE_PASSAGE_A_NIVEAU_A_SERVO V1.1__"	; Texte visible dans le '.hex'.
		dt			"__Le 19/05/2021___"
		dt			"__http://www.la-tour.info/uts/uts_index.html__"

;***************************************************************************************************************
; Boucle principale

boucle	nop
		nop
		nop
		nop
		btfss		bit_etat_cal
		goto		boucle

; On passe ici dans la phase de temps mort, quand phase = 2, et une seul fois quand le bit "bit_etat_cal" = 1.
; On en profite pour faire la lecture CAN et les calculs de toutes les variables.
; On a 15 msec pour tout faire.

		movlw		.255			; On temporise 8 msec avant d'aller lire les potentiomtres,
		movwf		temp1_L			; te temps que le pic de courant demand par les servomoteur, soit attnu.
		movlw		.20				; Tout ce sous-programme est parcouru en moins de 12 msec au total, y compris ces 8 msec de tempo.
		movwf		temp1_H
dl_1	decfsz		temp1_L,f
		goto		dl_1
		decfsz		temp1_H,f
		goto		dl_1

		bcf 		bit_etat_cal	; On ne passe qu'une fois toutes les 20 msec dans cette partie.
		bsf 		bit_etat_OV		; bit_etat_OV : 1 = Train dtect (Si entre == 0).
		btfsc		RAtrain
		bcf 		bit_etat_OV		; bit_etat_OV : 0 = Pas de train dtect (Si entre == 0).
		bsf 		bit_etat_TSV	; bit_etat_TSV : 0 = Servo 90, 1 = servo 180.
		btfsc		RBtypesrv		; Pas de cavalier = +5 volts = 1 => servo 90 = 0.
		bcf 		bit_etat_TSV
		bsf 		bit_etat_CLI	; Type de clignotement, 50% ou 40%.
		btfsc		RBtypecli		; Pas de cavalier = +5V => 50%.
		bcf 		bit_etat_CLI	; Clignotement : 1 => cycle 40% ou 0 =< cycle 50%.

		movf		cpt_etat,w
		btfss		STATUS,Z
		goto		etat_01 
; cpt_etat == 0. Etat habituel. Les barrires sont leves.
; On attend le passage du train.
		bcf			bit_etat_OVT
		btfss		bit_etat_OV
		goto		etat_20			; bit_etat_OV == 0 => Pas de train.
		incf		cpt_etat,f		; bit_etat_OV == 1 => Train dtect => cpt_etat == 1.
		movf		POT_DB,w
		btfss		bit_etat_BRO	
		movlw		.5				; Si la barrire n'tait pas compltement ouverte, donc en cours d'ouverture, on la referme pratiquement immdiatement.
		movwf		cpt_DB			; Arme la temporisation avant la fermeture des barrires.
		movf		POT_DO,w
		movwf		cpt_DO			; Arme la temporisation de dtection continu du train.
		bsf			bit_etat_LED
		movlw		.1				; Mise  jour au 19/05/2021 : 21 -> 1.
		movwf		cpt_led
		bcf			bit_etat_CL2
		bsf			bit_etat_SON	; Active les leds et la sonnerie.
		bcf			bit_etat_BRO	; Raz indicateur d'ouverture totale des barrires.
		goto		etat_20

; cpt_etat == 1. Le train est dtect.
; Les barrires vont bientt se fermer au bout de la tempo cpt_DB,
; sauf si l'on ne dtecte plus de train.
etat_01	movf		cpt_etat,w
		addlw		-1
		btfss		STATUS,Z
		goto		etat_02

		btfss		bit_etat_OV
		goto		etat01b
		movf		POT_DO,w		; bit_etat_OV : 1 = Train toujours dtect (Si entre == 0).
		movwf		cpt_DO			; Rarme la temporisation.

etat01b	decf		cpt_DO,f
		btfss		STATUS,Z
		goto		etat01c
		clrf		cpt_etat		; Plus de train dtect pendant toute la dure de la temporisation => cpt_etat == 0.
		bcf			bit_etat_LED	; Plus de clignotement, plus de sonnerie.
		bcf			bit_etat_SON
		bcf			bit_etat_OVT
		goto		etat_20

etat01c	decf		cpt_DB,f
		btfss		STATUS,Z
		goto		etat_20
		bsf			bit_etat_OVT	; Fin de la tempo, on peut fermer les barrires.
		bcf			bit_etat_BRF	; Raz indicateur de fermeture totale des barrires.
		bcf			bit_etat_BRO	; Raz indicateur d'ouverture totale des barrires.
		clrf		cp2_A_L			; Pour une fermeture progressive des barrires.
		clrf		cp2_A_H
		clrf		cp2_B_L
		clrf		cp2_B_H
		incf		cpt_etat,f		; En cours de fermeture. cpt_etat == 2.
		goto		etat_20			; Les leds continuent de clignoter, sonnerie toujours active.

; cpt_etat == 2. Le train est dtect. Les barrires sont en cours de fermeture,
; sauf si l'on ne dtecte plus de train.
etat_02	movf		cpt_etat,w
		addlw		-2
		btfss		STATUS,Z
		goto		etat_03

		btfss		bit_etat_OV
		goto		etat02b
		movf		POT_DO,w		; bit_etat_OV : 1 = Train toujours dtect (Si entre == 0).
		movwf		cpt_DO			; Rarme la temporisation.

etat02b	decf		cpt_DO,f
		btfss		STATUS,Z
		goto		etat02c

		clrf		cpt_etat		; Plus de train dtect pendant toute la dure de la temporisation => cpt_etat == 0.
		bcf			bit_etat_LED	; Plus de clignotement des leds, plus de sonnerie.
		bcf			bit_etat_SON
		bcf			bit_etat_OVT	; On peut ouvrir les barrires.
		bcf			bit_etat_BRO	; Raz indicateur d'ouverture totale des barrires.
		clrf		cp2_A_L			; Pour une ouverture progressive des barrires.
		clrf		cp2_A_H
		clrf		cp2_B_L
		clrf		cp2_B_H
		goto		etat_20

etat02c	btfss		bit_etat_BRF
		goto		etat_20
		incf		cpt_etat,f		; Les barrires sont compltement fermes. cpt_etat == 3.
		bcf			bit_etat_SON	; Plus de sonnerie.
		goto		etat_20

; cpt_etat == 3. Les barrires sont compltement fermes.
; On attend la fin de dtection du train.
etat_03	movf		cpt_etat,w
		addlw		-3
		btfss		STATUS,Z
		goto		etat_20

		btfss		bit_etat_OV
		goto		etat03b
		movf		POT_DO,w		;; bit_etat_OV : 1 = Train toujours dtect (Si entre == 0).
		movwf		cpt_DO			; Rarme la temporisation.

etat03b	decf		cpt_DO,f
		btfss		STATUS,Z
		goto		etat_20

		clrf		cpt_etat		; Plus de train dtect pendant toute la dure de la temporisation => cpt_etat == 0.
		bcf			bit_etat_LED	; Plus de clignotement des leds, plus de sonnerie.
		bcf			bit_etat_SON
		bcf			bit_etat_OVT	; On peut ouvrir les barrires.
		bcf			bit_etat_BRF	; Raz indicateur de fermeture totale des barrires.
		clrf		cp2_A_L			; Pour une ouverture progressive des barrires.
		clrf		cp2_A_H
		clrf		cp2_B_L
		clrf		cp2_B_H
		goto		etat_20

;***************************************************************************************************************
etat_20	call		lecture_etat_OV	; Pour recalculer les tats "etat_OVA/B", aprs laboration de etat_OVT.

		btfss		bit_etat_SON
		bcf			RCsonnerie		; bit_etat_SON == 0 => Arrt sonnerie.
		btfsc		bit_etat_SON
		bsf			RCsonnerie		; bit_etat_SON == 1 => Activation sonnerie.

		btfss		bit_etat_LED	; bit_etat_LED == 1 => Clignotement altern des leds demand.
		goto		led_06

		btfsc		bit_etat_CLI	; 1 / Clignotement : 1 = cycle 40% ou 0 = cycle 50% ?
		goto		led_03
		decfsz		cpt_led,f		; Clignotement cycle de 50%.
		goto		led_10			; cpt_led # 0.
		btfsc		bit_etat_CL2	; Clignotement bit_etat_CL2 = 0 => LED A ou bit_etat_CL2 = 1 => LED B.
		goto		led_01
		bsf			RCledA
		bcf			RCledB
		goto		led_02
led_01	bcf			RCledA
		bsf			RCledB
led_02	movlw		.21
		movwf		cpt_led
		movlw		WRbeCL2
		xorwf		PRbeCL2,f		; On inverse l'tat du bit "bit_etat_CL2".
		goto		led_10
									; 2 / Clignotement cycle de 40%.
led_03	btfss		bit_etat_CL2	; Clignotement bit_etat_CL2 = 0 => LED A ou bit_etat_CL2 = 1 => LED B.
		goto		led_04
		decf		cpt_led,w		; w = 20,19,18,,,3,2,1,0.
		sublw		3				; LED_A. w = 20,19,18,,,3,2,1,0.
		btfss		STATUS,C
		bsf			RCledA			; Compteur de 20  3 = Led A allume 18 fois sur 21.
		btfsc		STATUS,C
		bcf			RCledA			; Compteur de 0  2 = Led A teinte 3 fois sur 21.
		goto		led_05

led_04	decf		cpt_led,w		; w = 20,19,18,,,3,2,1,0.
		sublw		3				; LED_B. w = 20,19,18,,,3,2,1,0.
		btfss		STATUS,C
		bsf			RCledB			; Compteur de 20  3 = Led B allume 18 fois sur 21.
		btfsc		STATUS,C
		bcf			RCledB			; Compteur de 0  2 = Led B teinte 3 fois sur 21.

led_05	decfsz		cpt_led,f		; Clignotement  1,1 Hz = 830 msec = 2 * 416 msec = 2 * 20 * 20 msec.
		goto		led_10			; cpt_led # 0.
		movlw		.21				; cpt_led == 0.
		movwf		cpt_led
		movlw		WRbeCL2
		xorwf		PRbeCL2,f		; On inverse l'tat du bit "bit_etat_CL2".
		goto		led_10

led_06	decfsz		cpt_led,f		; On laisse les leds allumes jusqu' la fin du dernier clignotement complet.
		goto		led_10
		bcf			RCledA
		bcf			RCledB

;***************************************************************************************************************
; 	Dure total sspg = 1,7 msec.
; En retour, les variables suivantes sont renseignes :
;   POT_A1_HL = 13000  33460  Consignes
;   POT_A2_HL = 45045  65065
;   POT_B1_HL = 13000  33460
;   POT_B2_HL = 45045  65065
;   POT_VO_HL = 25  537  Vitesse ouverture
;   POT_VF_HL = 25  537  Vitesse fermeture
;   POT_DB    = 1  255  Temporisation pour la fermeture des barrires
;   POT_DO    = 1  255  Temporisation pour l'ouverture des barrires  la dtection de la fin du train
;   cpt_A_HL  = 60  1282  Calcul du pas unitaire pour les consignes.
;   cpt_B_HL  = 60  1282
;
; Ouverture/fermeture progressive, avec la formule suivante :
; cp2_A_LH = (7*cp2_A_LH + cpt_A_LH + 7)/8.   cpt_A = 2100 maximum donc pas de dbordement sur 16 bits.

led_10	call		lecture_tout_can 
		movf		cp2_A_L,w
		movwf		a1
		movf		cp2_A_H,w
		movwf		a2
		movlw		.7
		movwf		b1
		movlw		.0
		movwf		b2
		call		Multiply16x16bit ; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
		movf		cpt_A_L,w		;
		addwf		res1,f
		btfsc		STATUS,C
		incf		res2,f			;
		movf		cpt_A_H,w
		addwf		res2,f			;
		movf		res1,w
		movwf		cp2_A_L
		movf		res2,w
		movwf		cp2_A_H

		movlw		.7
		addwf		cp2_A_L,f			;
		btfsc		STATUS,C
		incf		cp2_A_H,f			;

		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			cp2_A_H,f
		rrf			cp2_A_L,f
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			cp2_A_H,f
		rrf			cp2_A_L,f
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			cp2_A_H,f
		rrf			cp2_A_L,f		; /8

; cp2_B_LH = (7*cp2_B_LH + cpt_B_LH + 7)/8.   cpt_B = 2100 maximum donc pas de dbordement sur 16 bits.
		movf		cp2_B_L,w
		movwf		a1
		movf		cp2_B_H,w
		movwf		a2
		movlw		.7
		movwf		b1
		movlw		.0
		movwf		b2
		call		Multiply16x16bit ; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
		movf		cpt_B_L,w		;
		addwf		res1,f
		btfsc		STATUS,C
		incf		res2,f			;
		movf		cpt_B_H,w
		addwf		res2,f			;
		movf		res1,w
		movwf		cp2_B_L
		movf		res2,w
		movwf		cp2_B_H

		movlw		.7
		addwf		cp2_B_L,f			;
		btfsc		STATUS,C
		incf		cp2_B_H,f			;

		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			cp2_B_H,f
		rrf			cp2_B_L,f
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			cp2_B_H,f
		rrf			cp2_B_L,f
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			cp2_B_H,f
		rrf			cp2_B_L,f		; /8

;***************************************************************************************************************
; SERVO_A
		btfsc		bit_etat_OVA
		goto		c_a_10
; bit_etat_OVA == 0 !  Consigne_A_HL -----> POT_A1.
;	Si consigne_A_HL == POT_A1  =>  On ne faire rien.
;	Si consigne_A_HL  > POT_A1  =>  consigne_A_HL = consigne_A_HL - cp2_A_HL
;	Si consigne_A_HL  < POT_A1  =>  consigne_A_HL = POT_A1 (minimum)
		movf		consigne_A_H,w
		subwf		POT_A1_H,w		; Test POT_A1_H - consigne_A_H ?
		btfss		STATUS,Z
		goto		c_a_01			; Diffrence sur les msb, pas besoin de tester les lsb.

		movf		consigne_A_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_A1_L,w		; Test POT_A1_L - consigne_A_L ?
		btfss		STATUS,Z
		goto		c_a_01			; POT_A1_HL # consigne_A_HL !
		btfss		RAinversA		; Egalit Z==1! POT_A1_HL == consigne_A_HL !
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfsc		RAinversA
		bsf			bit_etat_BRO	; Les barrires sont compltement ouverte.
		goto		c_a_fin 		; Z==1! => Egalit, on ne bouge plus la consigne du servo A.

c_a_01	btfsc		STATUS,C 		; Si "consigne_A_HL < POT_A1" C=1, Si "consigne_A_HL > POT_A1" C=0.
		goto		c_a_03			; C==0! => On est plus bas que la consigne basse.

		movf		cp2_A_L,w		; C==0! "consigne_A_HL > POT_A1_H", Alors calcul  faire : - cp2_A_HL.
		subwf		consigne_A_L,f	; Calcul.
		movf		cp2_A_H,w
		btfss		STATUS,C
		addlw		.1
		subwf		consigne_A_H,f	; consigne_A_HL = consigne_A_HL - cp2_A_HL
		movf		consigne_A_H,w	; Une fois le calcul fait, test POT_A1_H - consigne_A_H ?
		subwf		POT_A1_H,w
		btfss		STATUS,Z
		goto		c_a_02			; Diffrence sur les msb, pas besoin de tester les lsb.
		movf		consigne_A_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_A1_L,w		; Une fois le calcul fait, test POT_A1_L - consigne_A_L ?
		btfsc		STATUS,Z
		goto		c_a_fin 		; Z==1! => Egalit, on ne bouge plus la consigne du servo A.
c_a_02	btfss		STATUS,C 		; Egalit Z=1, "consigne_A_HL < POT_A1" C=1, "consigne_A_HL > POT_A1" C=0.
		goto		c_a_fin			; On est plus haut que la consigne basse, = OK.

c_a_03	movf		POT_A1_L,w		; C==0! "consigne_A_HL < POT_A1" alors consigne_A_HL = POT_A1.
		movwf		consigne_A_L	; On est plus bas que la consigne basse.
		movf		POT_A1_H,w		; On impose consigne = valeur de POT_A1.
		movwf		consigne_A_H
		btfss		RAinversA
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfsc		RAinversA
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.
		goto		c_a_fin

; bit_etat_OVA == 1 !  Consigne_A_HL -----> POT_A2.
;	Si consigne_A_HL == POT_A2  =>  On ne faire rien.
;	Si consigne_A_HL  < POT_A2  =>  consigne_A = consigne_A + cp2_A_HL
;	Si consigne_A_HL  > POT_A2 	=>  consigne_A = POT_A2 (maximum)
c_a_10	movf		consigne_A_H,w
		subwf		POT_A2_H,w		; Test POT_A2_H - consigne_A_H ?
		btfss		STATUS,Z
		goto		c_a_12			; Diffrence sur les msb, pas besoin de tester les lsb.
		movf		consigne_A_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_A2_L,w		; Test POT_A2_L - consigne_A_L ?
		btfss		STATUS,Z
		goto		c_a_12			; POT_A2_HL # consigne_A_HL !
		btfsc		RAinversA		; Egalit Z==1! POT_A2_HL == consigne_A_HL !
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfss		RAinversA
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.
		goto		c_a_fin			; Z==1! => Egalit, on ne bouge plus la consigne du servo A.

c_a_12	btfss		STATUS,C		; Si "consigne_A_HL < POT_A2" C=1, Si "consigne_A_HL > POT_A2" C=0.
		goto		c_a_14			; C==0! => On est plus haut que la consigne haute.
		movf		cp2_A_L,w		; C==1! => "consigne_A_HL < POT_A2_H", Alors calcul  faire : + cp2_A_HL.
		addwf		consigne_A_L,f	; Calcul.
		btfsc		STATUS,C
		incf		consigne_A_H,f
		btfsc		STATUS,Z		; STATUS-Z du incf.
		goto		c_a_14			; La consigne "consigne_A_H++" a dpass 65535, alors Consigne = POT_A2.
		movf		cp2_A_H,w
		addwf		consigne_A_H,f	; consigne_A_HL = consigne_A_HL + cp2_A_HL
		btfsc		STATUS,C
		goto		c_a_14			; La consigne "consigne_A_H++" a dpass 65535, alors Consigne = POT_A2.
		movf		consigne_A_H,w
		subwf		POT_A2_H,w		; Test POT_A2_H - consigne_A_H ?
		btfss		STATUS,Z
		goto		c_a_13			; Diffrence sur les msb, pas besoin de tester les lsb.
		movf		consigne_A_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_A2_L,w		; Test POT_A2_L - consigne_A_L ?
		btfsc		STATUS,Z
		goto		c_a_fin 		; Z==1 => Egalit, on ne bouge plus la consigne du servo A.
c_a_13	btfsc		STATUS,C 		; 
		goto		c_a_fin			; Consigne < 65535! => Alors goto fin.

c_a_14	movf		POT_A2_L,w		; La consigne_A_HL ne peut pas dpasser POT_A2_HL = Barrires compltement fermes.
		movwf		consigne_A_L	; On est plus haut que la consigne haute.
		movf		POT_A2_H,w		; On impose consigne = valeur de POT_A2.
		movwf		consigne_A_H
		btfsc		RAinversA
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfss		RAinversA
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.

; Calculs des consignes  envoyer sur les servo.
; SERVOA => consigne_A_HL = consigne_A_HL/13 pour placer cette valeur directement dans le registre CCP1.
c_a_fin	movf		consigne_A_L,w
		movwf		aargb2
		movf		consigne_A_H,w
		movwf		aargb1			; aargb1:aargb2 = POT_A1_HL ou POT_A2_HL.
		clrf		aargb0			; aargb0 = 0. Division par 13.
		movlw		.13
		movwf		bargb1
		movlw		.0
		movwf		bargb0
		call		fxd2416u		; Division de 24 bit  by 16 bit  to 24 bit.
									; Dividend = aargb0:aargb1:aargb2 (0 - most significant!) Divisor = bargb0:bargb1
		movf		aargb2,w
		movwf		cons_A_L13
		movf		aargb1,w
		movwf		cons_A_H13		; cons_A_LH13.

;***************************************************************************************************************
; SERVO_B
		btfsc		bit_etat_OVB
		goto		c_b_10
; bit_etat_OVB == 0 !  Consigne_B_HL -----> POT_B1.
;	Si consigne_B_HL == POT_B1  =>  On ne faire rien.
;	Si consigne_B_HL  > POT_B1  =>  consigne_B_HL = consigne_B_HL - cp2_B_HL
;	Si consigne_B_HL  < POT_B1  =>  consigne_B_HL = POT_B1 (minimum)
		movf		consigne_B_H,w
		subwf		POT_B1_H,w		; Test POT_B1_H - consigne_B_H ?
		btfss		STATUS,Z
		goto		c_b_01			; Diffrence sur les msb, pas besoin de tester les lsb.

		movf		consigne_B_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_B1_L,w		; test POT_B1_L - consigne_B_L ?
		btfss		STATUS,Z
		goto		c_b_01			; POT_B1_HL # consigne_B_HL !
		btfss		RAinversB		; Egalit Z==1! POT_B1_HL == consigne_B_HL !
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfsc		RAinversB
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.
		goto		c_b_fin 		; Z==1! => Egalit, on ne bouge plus la consigne du servo B.

c_b_01	btfsc		STATUS,C 		; Si "consigne_B_HL < POT_B1" C=1, Si "consigne_B_HL > POT_B1" C=0.
		goto		c_b_03			; C==0! => On est plus bas que la consigne basse.

		movf		cp2_B_L,w		; C==0! "consigne_B_HL > POT_B1_H", Alors calcul  faire : - cp2_B_HL.
		subwf		consigne_B_L,f	; Calcul.
		movf		cp2_B_H,w
		btfss		STATUS,C
		addlw		.1
		subwf		consigne_B_H,f	; consigne_B_HL = consigne_B_HL - cp2_B_HL
		movf		consigne_B_H,w	; Une fois le calcul fait, test POT_B1_H - consigne_B_H ?
		subwf		POT_B1_H,w		
		btfss		STATUS,Z
		goto		c_b_02			; Diffrence sur les msb, pas besoin de tester les lsb.
		movf		consigne_B_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_B1_L,w		; Une fois le calcul fait, test POT_B1_L - consigne_B_L ?
		btfsc		STATUS,Z
		goto		c_b_fin 		; Z==1: => Egalit, on ne bouge plus la consigne du servo B.
c_b_02	btfss		STATUS,C 		; Egalit Z=1, "consigne_B_HL < POT_B1" C=1, "consigne_B_HL > POT_B1" C=0.
		goto		c_b_fin			; On est plus haut que la consigne basse, = OK.

c_b_03	movf		POT_B1_L,w		; C==0! "consigne_B_HL < POT_B1" alors consigne_B_HL = POT_B1.
		movwf		consigne_B_L	; On est plus bas que la consigne basse.
		movf		POT_B1_H,w		; On impose consigne = valeur de POT_B1.
		movwf		consigne_B_H
		btfss		RAinversB
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfsc		RAinversB
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.
		goto		c_b_fin

; bit_etat_OVB == 1 !  Consigne_B_HL -----> POT_B2.
;	Si consigne_B_HL == POT_B2  =>  On ne faire rien.
;	Si consigne_B_HL  < POT_B2  =>  consigne_B = consigne_B + cp2_B_HL
;	Si consigne_B_HL  > POT_B2 	=>  consigne_B = POT_B2 (maximum)
c_b_10	movf		consigne_B_H,w
		subwf		POT_B2_H,w		; Test POT_B2_H - consigne_B_H ?
		btfss		STATUS,Z
		goto		c_b_12			; Diffrence sur les msb, pas besoin de tester les lsb.
		movf		consigne_B_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_B2_L,w		; Test POT_B2_L - consigne_B_HL ?
		btfss		STATUS,Z
		goto		c_b_12			; POT_B2_HL # consigne_B_HL !
		btfsc		RAinversB		; Egalit Z==1! POT_B2_HL == consigne_B_HL !
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfss		RAinversB
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.
		goto		c_b_fin			; Z==1! => Egalit, on ne bouge plus la consigne du servo B.

c_b_12	btfss		STATUS,C		; Si "consigne_B_HL < POT_B2" C=1, Si "consigne_B_HL > POT_B2" C=0.
		goto		c_b_14			; C==1! => "consigne_B_HL < POT_B2_H", Alors calcul  faire : + cp2_B_HL.
		movf		cp2_B_L,w		; C=1. consigne_B_HL < POT_B2_H.  + cp2_B_HL.
		addwf		consigne_B_L,f	; calcul.
		btfsc		STATUS,C
		incf		consigne_B_H,f
		btfsc		STATUS,Z		; STATUS-Z du incf.
		goto		c_b_14			; La consigne "consigne_B_H++" a dpass 65535, alors Consigne = POT_B2.
		movf		cp2_B_H,w
		addwf		consigne_B_H,f	; Test consigne_B_HL = consigne_B_HL + cp2_B_HL ?
		btfsc		STATUS,C
		goto		c_b_14			; La consigne "consigne_B_H++" a dpass 65535, alors Consigne = POT_B2.
		movf		consigne_B_H,w
		subwf		POT_B2_H,w		; Test POT_B2_H - consigne_B_H ?
		btfss		STATUS,Z
		goto		c_b_13			; Diffrence sur les msb, pas besoin de tester les lsb.
		movf		consigne_B_L,w	; Egalit des msb, il faut alors tester les lsb.
		subwf		POT_B2_L,w		; Test POT_B2_L - consigne_B_L ?
		btfsc		STATUS,Z
		goto		c_b_fin 		; Z==1 => Egalit, on ne bouge plus la consigne du servo B.
c_b_13	btfsc		STATUS,C 		; 
		goto		c_b_fin			; Consigne < 65535! => Alors goto fin.

c_b_14	movf		POT_B2_L,w		; La consigne_B_HL ne peut pas dpasser POT_B2_HL = Barrires compltement fermes.
		movwf		consigne_B_L	; On est plus haut que la consigne haute.
		movf		POT_B2_H,w		; On impose consigne = valeur de POT_B2.
		movwf		consigne_B_H
		btfsc		RAinversB
		bsf			bit_etat_BRF	; Les barrires sont compltement fermes.
		btfss		RAinversB
		bsf			bit_etat_BRO	; Les barrires sont compltement ouvertes.

; Calculs des consignes  envoyer sur les servo.
; SERVOB => consigne_B_HL = consigne_B_HL/13 pour placer cette valeur directement dans le registre CCP1.
c_b_fin	movf		consigne_B_L,w
		movwf		aargb2
		movf		consigne_B_H,w
		movwf		aargb1			; aargb1:aargb2 = POT_B1_HL ou POT_B2_HL.
		clrf		aargb0			; aargb0 = 0. Division par 13.
		movlw		.13
		movwf		bargb1
		movlw		.0
		movwf		bargb0
		call		fxd2416u		; Division de 24 bit  by 16 bit  to 24 bit.
									; Dividend = aargb0:aargb1:aargb2 (0 - most significant!) Divisor = bargb0:bargb1
		movf		aargb2,w
		movwf		cons_B_L13
		movf		aargb1,w
		movwf		cons_B_H13		; cons_B_LH13.

;***************************************************************************************************************
tstpo30
; Permet un espacement des impulsions de 20 msec, dans la phase == 2.
; Temps mort = 20 msec - consigne servo A - consigne servo B = 15  19 msec.
;  tempsmort = 400000 - consigne_A - consigne_B.
		movlw		0x40				; 20 msec = 40000 = 0x9C40
		movwf		tpmort_L13
		movlw		0x9C
		movwf		tpmort_H13			; tpmort_HL13 = 400000

		movf		cons_A_L13,w
		subwf		tpmort_L13,f
		movf		cons_A_H13,w
		btfss		STATUS,C
		addlw		.1
		subwf		tpmort_H13,f

		movf		cons_B_L13,w
		subwf		tpmort_L13,f
		movf		cons_B_H13,w
		btfss		STATUS,C
		addlw		.1
		subwf		tpmort_H13,f		; tempsmort = 400000 - cons_A_HL13 - cons_B_HL13

		goto		boucle

;***************************************************************************************************************
; Partie interruption
;	Interruption sur comparaison Timer1 == CCP1 uniquement.

interrup
		bcf 		INTCON,GIE		; Inhibe toutes les interruptions.
		movwf   	w_temp  		; Sauver registre w.
		swapf		STATUS,w		; Swap status avec rsultat dans w.
		movwf		status_temp		; Sauver status swapp.
		BANK0

		btfss		PIR1,CCP1IF		; Interrupt comparaison Timer1 == CCP1 en cours ?
		goto		restreg			; Non !
		bcf 		T1CON, TMR1ON	; Oui => Arrt du Timer1.
		call		int_timer1_0	; Traiter interrupt comparaison Timer1 == CCP1.
		clrf		TMR1L
		clrf		TMR1H
		bsf 		T1CON, TMR1ON	; Mise en marche du Timer1.
		bcf 		PIR1,CCP1IF		; Effacer flag interupt comparaison Timer1.

restreg	swapf		status_temp,w	; Swap ancien status, rsultat dans w.
		movwf		STATUS			; Restaurer status.
		swapf		w_temp,f		; Inversion L et H de l'ancien W sans modifier Z.
		swapf		w_temp,w  		; Rinversion de L et H dans W, W restaur sans modifier status.
		retfie  					; Fin routine d'interruption.

;***************************************************************************************************************
; Routine d'interruption.
;   La comparaison Timer1 == CCP1 est arrive  galit.
;   phase = 0, 1, 2 suivant le traitement de l'impulsion en cours de traitement.
int_timer1_0
		movf		phase,w
		btfss		STATUS,Z
		goto		int_timer1_1
; Phase == 0
;   Crneau SERVO A
;   Consigne_A_HL13 => CCPR1_HL
		movf		cons_A_L13,w
		movwf		CCPR1L
		movf		cons_A_H13,w
		movwf		CCPR1H
		bsf 		RCservoA		; Sortie Servo A = 1.
		incf		phase,f			; Phase = 1.

;;; LIGNES SUIVANTES POUR DEBUG UNIQUEMENT.
;;; Permet d'afficher sur l'oscilloscope, temp1_HL.
;;; Pour traiter les cas de tremblement sur les lectures CAN.
IF 0
		bsf 		RCservoA
		movlw		.60
		movwf		temp1
		decfsz		temp1,f
		goto		$-1
		movf		zz_L,w
		movwf		temp1_L
		movf		zz_H,w
		movwf		temp1_H
		movlw		.17
		movwf		a1
aaaa	movlw		.50
		movwf		temp1
		decfsz		temp1,f
		goto		$-1
		bsf 		RCservoA		
		nop
		nop
		nop
		nop
		nop
		nop
		bcf 		RCservoA		
		btfsc		temp1_H,7
		bsf 		RCservoA		
		btfss		temp1_H,7
		bcf 		RCservoA		
		bcf			STATUS,C
		rlf			temp1_L
		rlf			temp1_H
		decfsz		a1,f
		goto		aaaa
		bcf 		RCservoA
ENDIF	
		return

int_timer1_1
		movf		phase,w
		addlw		-1
		btfss		STATUS,Z
		goto		int_timer1_2
; Phase == 1
;   Crneau SERVO B
;   Consigne_B_HL13 => CCPR1_HL
		bcf 		RCservoA		; Sortie Servo A = 0.
		movf		cons_B_L13,w
		movwf		CCPR1L
		movf		cons_B_H13,w
		movwf		CCPR1H
		bsf 		RCservoB		; Sortie Servo B = 1.
		incf		phase,f			; Phase = 2.
		return

int_timer1_2
		movf		phase,w
		addlw		-2
		btfss		STATUS,Z
		goto		int_timer1_3
; Phase == 2
;   Crneau de 15  19 msec de libre, avant de renvoyer le crneau du servo A.
;   tempsmort = 20(msec) - Consigne servo A - Consigne servo B
;   tempsmort => CCPR1
		bcf 		RCservoB		; Sortie Servo B = 0.
		movf		tpmort_L13,w
		movwf		CCPR1L
		movf		tpmort_H13,w
		movwf		CCPR1H
		clrf		phase			; Phase = 0.
		bsf 		bit_etat_cal	; 1 = Demande de lecture CAN et des calculs de toutes les variables, dans la boucle principale.
		return						; On a 15 msec pour tout faire.

int_timer1_3
; Phase >= 3, rien de prvu pour le moment.
		return

;***************************************************************************************************************
; Code d'initialisation du PIC  la mise sous tension.
init	bcf 		INTCON,GIE		; Pas d'interruption pour le moment.
		BANK0
		bcf 		INTCON,GIE

		movlw		0x20
		movwf		FSR				; on charge l'adresse de dbut.
init_01	clrf		INDF 			; effacement du contenu du registre point par FSR
		incf		FSR,f
		movlw		0x7F+.1			; adresse de fin + 1.)
		subwf		FSR, W
		btfss		STATUS,Z
		goto		init_01

		BANK0
		clrf		PORTA
		clrf		PORTB
		clrf		PORTC
		clrf		INTCON			; Pas d'interruption pour le moment.
		movlw		b'10000001'     ; Format de conversion analogique 12 bits cadr  droite sur deux deux octets + Vref=+5V + Convertisseur=actif.
		movwf		ADCON0			; ADFM = 1, ADON = 1.
		movlw		b'00001010'		; Utilisation CCP1M = 1010 = Compare mode, generate software interrupt on match (CCP1IF bit is set, CCP1 pin is unaffected)
		movwf		CCP1CON

		BANK1
		movlw		b'01110000'		; 'x111' = Vitesse 8 Mhz! + 'xxx' + '0'=SCS interne.
		movwf		OSCCON
		movlw		b'01100000'		; 'x010' = Dure de conversion analogique = 4s => F/64  8Mhz + 'xxxx'.
		nop
		nop
		movwf		ADCON1
		movlw		b'00100000'		; '1' = Weak pull-up actif sur RA5.
		movwf		WPUA

		BANK2
		movlw		b'11000000'     ; '1' = Weak pull-up actif sur RB7 + RB6.
		movwf		WPUB

		BANK1
		movlw		b'00000000'     ; '0' = PortA+B rsistances de rappel +  Timer0 Clock Source Select bit,
		movwf		OPTION_REG		; Prescaler is assigned to the Timer0 module.
		clrf		PIE1			; '0' = Pas d'interruption.
		clrf		PIE2
		clrf		IOCA

		BANK0
		clrf		PIR1
		clrf		PIR2

; RA0 - CAN0  = POT_DO, Temporisation de l'ouverture des barrires, en absence de signal de train.
; RA1 - CAN1  = POT_DB, Temporisation de la fermeture des barrires, aprs le clignotement.
; RA2 - CAN2  = POT_VF, Vitesse de fermeture, 0,5  10 secondes.
; RA3 - IN  = Entre de dtection du train = Prsence train = 0 volt.
; RA4 - IN  = Inversion de la course du Servo_A, inversion = cavalier en place.
; RA5 - IN  = Cavalier libre.
;---------------
; RB4 - CAN10 = POT_A2, Servo_A, position ferm POT_TB.
; RB5 - CAN11 = POT_A1, Servo_A, position ouvert  POT_DE.
; RB6 - IN  = Servo 90 ou 180, cavalier en place = 180.
; RB7 - IN  = Inversion de la course du Servo_B, inversion = cavalier en place.
;---------------
; RC0 - CAN4  = POT_VO, Vitesse d'ouverture, 0,5  10 secondes.
; RC1 - CAN5  = POT_B2, Servo_B, position ferm POT_VO,
; RC2 - CAN6  = POT_B1, Servo_B, position ouvert POT_VF,
; RC3 - OUT = Sortie vers SERVO_B.
; RC4 - OUT = Sortie vers SERVO_A.
; RC5 - OUT = Sortie activation sonnerie.
; RC6 - OUT = Sortie vers LED_A
; RC7 - OUT = Sortie vers LED_B
;
		BANK1
		movlw		b'11111111'		; '1' = Entre.
		movwf		TRISA
		movlw		b'11111111'
		movwf		TRISB
		movlw		b'00000111'
		movwf		TRISC

		BANK2
		movlw		b'01110111'     ; 1 = Utilisation en entre analogique.
		movwf		ANSEL
		movlw		b'00001100'     ; 1 = Utilisation en entre analogique.
		movwf		ANSELH
		clrf		CM1CON0			; Les comparateurs ne sont pas utiliss.
		clrf		CM2CON0

		BANK0
		clrf		etat1
		clrf		etat2

		movlw		.255			; A la mise sous tension, on patiente une seconde avant de dmarrer,
		movwf		temp1			; pour avoir l'tat de l'entre "Dtection train" correct.
		movwf		temp1_L
		movlw		.20
		movwf		temp1_H
del_1	decfsz		temp1,f
		goto		del_1
		decfsz		temp1_L,f
		goto		del_1
		decfsz		temp1_H,f
		goto		del_1

		bsf 		bit_etat_OV		; bit_etat_OV  : 1 = Fermeture en cours (Si entre == 0).
		btfsc		RAtrain
		bcf 		bit_etat_OV		; bit_etat_OV  : 1 = Fermeture en cours (Si entre == 0).

		bsf 		bit_etat_OVT	; bit_etat_OVT  : 1 = Fermeture en cours (Si entre == 0).
		btfss		bit_etat_OV
		bcf 		bit_etat_OVT	; bit_etat_OVT  : 1 = Fermeture en cours (Si entre == 0).
		
		bsf 		bit_etat_TSV	; bit_etat_TSV : 0 = Servo 90, 1 = servo 180.
		btfsc		RBtypesrv		; Pas de cavalier = +5 volts = 1 => servo 90 = 0.
		bcf 		bit_etat_TSV

		bsf 		bit_etat_CLI	; Type de clignotement, 50% ou 40%.
		btfsc		RBtypecli		; Pas de cavalier = +5V = 50%.
		bcf 		bit_etat_CLI	; Clignotement : 1 = cycle 40% ou 0 = cycle 50%.
	
		call		lecture_etat_OV ; Pour calculer les tats "etat_OVA/B".
		movlw		.30
		movwf		cpt_etat		; La variable "cpt_etat" ne sert pas encore.
		call		lecture_tout_can ; Pour initialiser toutes les variables avant de commencer.
		decfsz		cpt_etat,f		; > 16 fois pour calculer la premire moyenne.
		goto		$-2

; On place les barrires en position ferme ou ouverte, suivant si un train est prsent et suivant l'tat des cavaliers.
;   bit_etat_OVT  == 1 => Fermeture des barrires en cours.
;   bit_etat_OVA  => 0 : Consigne_A = POT_A1.
;                    1 : Consigne_A = POT_A2. Cavalier inverseur SERVO_A en place.
;   bit_etat_OVB  => 0 : Consigne_B = POT_B1.
;                    1 : Consigne_B = POT_B2. Cavalier inverseur SERVO_B en place.
		movf		POT_A1_L,w
		btfsc		bit_etat_OVA
		movf		POT_A2_L,w
		movwf		aargb2
		movf		POT_A1_H,w
		btfsc		bit_etat_OVA
		movf		POT_A2_H,w
		movwf		aargb1			; aargb1:aargb2 = POT_A1_HL ou POT_A2_HL.
		movf		aargb2,w
		movwf		consigne_A_L
		movf		aargb1,w
		movwf		consigne_A_H	; Au dpart, consigne_A_HL = POT_A1 ou POT_A2.
		clrf		aargb0			; aargb0 = 0. Division par 13.
		movlw		.13
		movwf		bargb1
		movlw		.0
		movwf		bargb0
		call		fxd2416u		; Division de 24 bit  by 16 bit  to 24 bit.
									; Dividend = aargb0:aargb1:aargb2 (0 - most significant!) Divisor = bargb0:bargb1
		movf		aargb2,w
		movwf		cons_A_L13
		movf		aargb1,w
		movwf		cons_A_H13		; Au dpart, cons_A_HL13 = POT_A1/13 ou POT_A2/13.

		movf		POT_B1_L,w
		btfsc		bit_etat_OVB
		movf		POT_B2_L,w
		movwf		aargb2
		movf		POT_B1_H,w
		btfsc		bit_etat_OVB
		movf		POT_B2_H,w
		movwf		aargb1			; aargb1:aargb2 = POT_B1_HL ou POT_B2_HL.
		movf		aargb2,w
		movwf		consigne_B_L
		movf		aargb1,w
		movwf		consigne_B_H	; Au dpart, consigne_B_HL = POT_B1 ou POT_B2.
		clrf		aargb0			; aargb0 = 0. Division par 13.
		movlw		.13
		movwf		bargb1
		movlw		.0
		movwf		bargb0
		call		fxd2416u		; Division de 24 bit  by 16 bit  to 24 bit.
									; Dividend = aargb0:aargb1:aargb2 (0 - most significant!) Divisor = bargb0:bargb1
		movf		aargb2,w
		movwf		cons_B_L13
		movf		aargb1,w
		movwf		cons_B_H13		; Au dpart, cons_B_HL13 = POT_B1/13 ou POT_B2/13.

		movlw		0x70
		movwf		tpmort_L13		; tempsmort = 38000 = 0x9470
		movwf		CCPR1L
		movlw		0x94
		movwf		tpmort_H13		; tpmort_HL13 = 19 msec par dfaut au dmarrage pour la premire boucle.
		movwf		CCPR1H
		clrf		TMR1L
		clrf		TMR1H
		movlw		.3
		btfss		bit_etat_OV
		movlw		.0
		movwf		cpt_etat		; 0 ou 3. Etat au dpart, suivant si les barrires sont leves (0) ou baisses (3).

		btfss		bit_etat_OV
		bsf			bit_etat_BRO	; Si pas de train => Set indicateur d'ouverture totale des barrires.
		btfss		bit_etat_OV
		goto		init_05			; Si pas de train => goto init_05.
		bsf			bit_etat_LED	; Si train => Clignotement des leds, pas de sonnerie.
		bsf			bit_etat_BRF	; Si train => Set indicateur de fermeture totale des barrires.
		movlw		.1
		movwf		cpt_led			; Mise  jour au 19/05/2021 : 21 -> 1.
		bcf			bit_etat_CL2

init_05	bsf 		bit_etat_cal	; 1 = Demande de lecture CAN et des calculs de toutes les variables au 1 er tour.
		movlw		.2
		movwf		phase			; On commence en phase = 2.

		clrf		cp2_A_L			; Pour une ouverture progressive des barrires.
		clrf		cp2_A_H
		clrf		cp2_B_L
		clrf		cp2_B_H

		bcf 		PIR1,CCP1IF		; Efface le Flag CCP1 Interrupt Enable bit.
		bsf 		T1CON, TMR1ON	; Mise en marche du Timer1.
		BANK1
		bsf 		PIE1,CCP1IE		; CCP1 Interrupt Enable bit
		BANK0
		bsf 		INTCON,PEIE
		bsf 		INTCON,GIE		; Enables all unmasked interrupts
		goto		boucle


;***************************************************************************************************************
; Sous programme de lecture des entres prsence train et position des cavaliers inverseurs.
; En retour, les variables suivantes sont renseignes :
;   bit_etat_OVT  == 1 => Fermeture des barrires en cours.
;   bit_etat_OVA  => 0 : Consigne_A = POT_A1.
;                    1 : Consigne_A = POT_A2. Cavalier inverseur SERVO_A en place.
;   bit_etat_OVB  => 0 : Consigne_B = POT_B1.
;                    1 : Consigne_B = POT_B2. Cavalier inverseur SERVO_B en place.
lecture_etat_OV
		btfsc		bit_etat_OVT
		goto		etat_o1
		bcf 		bit_etat_OVA
		btfss		RAinversA
		bsf 		bit_etat_OVA	; bit_etat_OVA : 0 => Consigne = POT_A1,    1 => Consigne = POT_A2.
		bcf 		bit_etat_OVB
		btfss		RAinversB
		bsf 		bit_etat_OVB	; bit_etat_OVB : 0 => Consigne = POT_B1,    1 => Consigne = POT_B2.
		goto		etat_o2
etat_o1	bsf 		bit_etat_OVA
		btfss		RAinversA
		bcf 		bit_etat_OVA
		bsf 		bit_etat_OVB
		btfss		RAinversB
		bcf 		bit_etat_OVB
etat_o2	return


;***************************************************************************************************************
; Sous programme de lecture CAN et de mise  l'chelle dans la foule. Temps d'xecution total = 1,7 msec.
; En retour, les variables suivantes sont renseignes :
;   POT_A1_HL = Consigne servo 90 26000  38276 ou consigne servo 180 : 13000  38575.
;   POT_A2_HL = Consigne servo 90 39720  51996 ou consigne servo 180 : 39720  65295.
;   POT_B1_HL = Consigne servo 90 26000  38276 ou consigne servo 180 : 13000  38575.
;   POT_B2_HL = Consigne servo 90 39720  51996 ou consigne servo 180 : 39720  65295.
;   POT_VO_HL = 25  537  Vitesse ouverture
;   POT_VF_HL = 25  537  Vitesse fermeture
;   POT_DB    = 1  255  Temporisation pour la fermeture des barrires
;   POT_DO    = 1  255  Temporisation pour l'ouverture des barrires  la dtection de la fin du train
;   cpt_A_HL  = 60  1282  Calcul du pas unitaire pour les consignes.
;   cpt_B_HL  = 60  1282
;
; RA0 - CAN0  = POT_DO, Temporisation de l'ouverture des barrires, en absence de signal de train
; RA1 - CAN1  = POT_DB, Temporisation de la fermeture des barrires, aprs le clignotement
; RA2 - CAN2  = POT_VF, Vitesse de fermeture, 0,5  10 secondes.
; RC0 - CAN4  = POT_VO, Vitesse d'ouverture, 0,5  10 secondes.
; RC1 - CAN5  = POT_B2, Servo_B, position ferm
; RC2 - CAN6  = POT_B1, Servo_B, position ouvert
; RB4 - CAN10 = POT_A2, Servo_A, position ferm
; RB5 - CAN11 = POT_A1, Servo_A, position ouvert

lecture_tout_can
		bcf 		ADCON0,CHS0		; '0000' = CAN0 = RA0 = POT_DO, Temporisation de l'ouverture des barrires, en absence de signal de train
		bcf 		ADCON0,CHS1
		bcf 		ADCON0,CHS2
		bcf 		ADCON0,CHS3
		call		lecture_can3	; En retour : temp1_H + temp1_L = 0  255.
		movf		temp1_L,w
		addlw		.3
		btfsc		STATUS,C
		movlw		.255
		movwf		POT_DO			; 3  255 POT_DB = 1  255 = 0,1  5 secondes.
		
; --------
		bsf 		ADCON0,CHS0		; '0001' = CAN1 = RA1 = POT_DB, Temporisation de la fermeture des barrires, aprs le clignotement
		call		lecture_can3	; En retour : temp1_H:temp1_L = 0  255.
		movf		temp1_L,w
		addlw		.3
		btfsc		STATUS,C
		movlw		.255
		movwf		POT_DB			; 3  255 POT_DB = 1  255 = 0,1  5 secondes.
; --------
		bcf 		ADCON0,CHS0		; '0010' = CAN2 = RA2 = POT_VF, Vitesse de fermeture, 0,5  10 secondes.
		bsf 		ADCON0,CHS1
		call		lecture_can2	; En retour : temp1_H:temp1_L = 0  511
		movf		temp1_L,w
		movwf		POT_VF_L
		movf		temp1_H,w
		movwf		POT_VF_H
		movlw		.25				; Minimum = 0,5 secondes
		addwf		POT_VF_L,f
		btfsc		STATUS,C
		incf		POT_VF_H,f		; POT_VF = 25  537
; --------
		bcf 		ADCON0,CHS0		; '0100' = CAN4 = RC0 = POT_VO, Vitesse d'ouverture, 0,5  10 secondes.
		bcf 		ADCON0,CHS1
		bsf 		ADCON0,CHS2
		call		lecture_can2	; En retour : temp1_H:temp1_L = 0  511
		movf		temp1_L,w
		movwf		POT_VO_L
		movf		temp1_H,w
		movwf		POT_VO_H
		movlw		.25				; Minimum = 0,5 secondes
		addwf		POT_VO_L,f
		btfsc		STATUS,C
		incf		POT_VO_H,f		; POT_VO = 25  537
; --------
		bsf 		ADCON0,CHS0		; '0101' = CAN5 = RC1 = POT_B2, Servo_B, position ferm
		btfss		RAinversB
		bcf 		ADCON0,CHS0		; '0110' inverse la lecture.
		btfss		RAinversB
		bsf 		ADCON0,CHS1		; '0110' inverse la lecture.
		call		lecture_can1	; En retour : temp1_H + temp1_L = 0  1023
		call		moy_can5		; Moyenne temp1_H:temp1_L sur les 16 dernires mesures.
		movlw		.12
		btfsc		bit_etat_TSV	; Suivant servo 90 (=0) ou 180 (=1) => = *12 ou = *25.
		movlw		.25
		movwf		a1
		movlw		.0
		movwf		a2
		movf		temp1_L,w
		movwf		b1
		movf		temp1_H,w
		movwf		b2
		call 		Multiply16x16bit ; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
		movf		res1,w
		movwf		POT_B2_L
		movf		res2,w
		movwf		POT_B2_H
		movlw		0x28			; +39720 = +0x9B28
		addwf		POT_B2_L,f
		btfsc		STATUS,C
		incf		POT_B2_H,f
		movlw		0x9B
		addwf		POT_B2_H,f		; POT_B2 = 39720 + 12(ou 25) * CAN = 39720  51996 ou 39720  65295.
; --------
		bcf 		ADCON0,CHS0		; '0110' = CAN6 = RC2 = POT_B1, Servo_B, position ouvert
		btfss		RAinversB
		bsf 		ADCON0,CHS0		; '0101' inverse la lecture.
		bsf 		ADCON0,CHS1
		btfss		RAinversB
		bcf 		ADCON0,CHS1		; '0101' inverse la lecture.
		call		lecture_can1	; En retour : temp1_H:temp1_L = 0  1023
		call		moy_can6		; Moyenne temp1_H:temp1_L sur les 16 dernires mesures.
		movlw		.12
		btfsc		bit_etat_TSV	; Suivant servo 90 (=0) ou 180 (=1) => = *12 ou = *25.
		movlw		.25
		movwf		a1
		movlw		.0
		movwf		a2
		movf		temp1_L,w
		movwf		b1
		movf		temp1_H,w
		movwf		b2
		call 		Multiply16x16bit ; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
		movf		res1,w
		movwf		POT_B1_L
		movf		res2,w
		movwf		POT_B1_H
		btfsc		bit_etat_TSV	; Suivant servo 90 (=0) ou 180 (=1) => = *12 ou = *25.
		goto		lec_01			; Suivant servo 90 (=0) ou 180 (=1) => = +26000 ou = +13000.
		movlw		0x90			; +26000 = +0x6590
		addwf		POT_B1_L,f
		btfsc		STATUS,C
		incf		POT_B1_H,f
		movlw		0x65
		addwf		POT_B1_H,f		;
		goto		lec_02
lec_01	movlw		0xC8			; +13000 = +0x32C8
		addwf		POT_B1_L,f
		btfsc		STATUS,C
		incf		POT_B1_H,f
		movlw		0x32
		addwf		POT_B1_H,f		; POT_B1 = 26000(ou 13000) + 12(ou 25) * CAN = 26000  38276 ou 13000  38575.

; --------
lec_02	bsf 		ADCON0,CHS1
		bcf 		ADCON0,CHS2		; '1010' = CAN10 = RB4 = POT_A2, Servo_A, position ferm
		bsf 		ADCON0,CHS3
		bcf 		ADCON0,CHS0		; '1010'
		btfss		RAinversA
		bsf 		ADCON0,CHS0		; '1011' = CAN11 = RB5 = POT_A1, Servo_A, position ouvert
		call		lecture_can1	; En retour : temp1_H:temp1_L = 0  1023
		call		moy_can10		; Moyenne temp1_H:temp1_L sur les 16 dernires mesures.
		movlw		.12
		btfsc		bit_etat_TSV	; Suivant servo 90 (=0) ou 180 (=1) => = *12 ou = *25.
		movlw		.25
		movwf		a1
		movlw		.0
		movwf		a2
		movf		temp1_L,w
		movwf		b1
		movf		temp1_H,w
		movwf		b2
		call 		Multiply16x16bit ; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
		movf		res1,w
		movwf		POT_A2_L
		movf		res2,w
		movwf		POT_A2_H
		movlw		0x28			; +39720 = +0x9B28
		addwf		POT_A2_L,f
		btfsc		STATUS,C
		incf		POT_A2_H,f
		movlw		0x9B
		addwf		POT_A2_H,f		; POT_A2 = 39720 + 12(ou 25) * CAN = 39720  51996 ou 39720  65295.
; --------
		bsf 		ADCON0,CHS0		; '1011' = CAN11 = RB5 = POT_A1, Servo_A, position ouvert
		btfss		RAinversA
		bcf 		ADCON0,CHS0		; '1010' = CAN10 = inverse la lecture.
		call		lecture_can1	; En retour : temp1_H:temp1_L = 0  1023
		call		moy_can11	; Moyenne temp1_H:temp1_L sur les 16 dernires mesures.
		movlw		.12
		btfsc		bit_etat_TSV	; Suivant servo 90 (=0) ou 180 (=1) => = *12 ou = *25.
		movlw		.25
		movwf		a1
		movlw		.0
		movwf		a2
		movf		temp1_L,w
		movwf		b1
		movf		temp1_H,w
		movwf		b2
		call 		Multiply16x16bit ; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
		movf		res1,w
		movwf		POT_A1_L
		movf		res2,w
		movwf		POT_A1_H
		btfsc		bit_etat_TSV	; Suivant servo 90 (=0) ou 180 (=1) => = *12 ou = *25.
		goto		lec_11			; Suivant servo 90 (=0) ou 180 (=1) => = +26000 ou = +13000.
		movlw		0x90			; +26000 = +0x6590
		addwf		POT_A1_L,f
		btfsc		STATUS,C
		incf		POT_A1_H,f
		movlw		0x65
		addwf		POT_A1_H,f		;
		goto		lec_12
lec_11	movlw		0xC8			; +13000 = +0x32C8
		addwf		POT_A1_L,f
		btfsc		STATUS,C
		incf		POT_A1_H,f
		movlw		0x32
		addwf		POT_A1_H,f		; POT_A1 = 26000(ou 13000) + 12(ou 25) * CAN = 26000  38276 ou 13000  38575.
; --------
; Calcul du pas unitaire.
; Pas incrmental = cpt_B_HL = (POTA2-POTA1)/(POT_VO ou POT_VF) = 22  1282 pour 0,5  10,7 secondes.
; SERVOA =>
lec_12	movf		POT_A1_L,w		; POT_A2 - POT_A1.
		subwf		POT_A2_L,w
		movwf		aargb2
		btfss		STATUS,C
		goto		$+6
		movf		POT_A1_H,w
		subwf		POT_A2_H,w
		movwf		aargb1
		decf		aargb1,f
		goto		$+4
		movf		POT_A1_H,w
		subwf		POT_A2_H,w
		movwf		aargb1

		clrf		aargb0
		movf		POT_VO_L,w
		btfsc		bit_etat_OV
		movf		POT_VF_L,w
		movwf		bargb1
		movf		POT_VO_H,w
		btfsc		bit_etat_OV
		movf		POT_VF_H,w
		movwf		bargb0
		call		fxd2416u		; Division de 24 bit  by 16 bit  to 24 bit.
									; Dividend = aargb0:aargb1:aargb2 (0 - most significant!) Divisor = bargb0:bargb1
		movf		aargb2,w
		movwf		cpt_A_L
		movf		aargb1,w
		movwf		cpt_A_H			; cpt_A_HL = 22  1282 pour 0,5  10,7 secondes.

; Pas incrmental = cpt_B_HL = (POTB2-POTB1)/(POT_VO ou POT_VF) = 22  1282 pour 0,5  10,7 secondes.
; SERVOB =>
		movf		POT_B1_L,w		; POT_B2 - POT_B1.
		subwf		POT_B2_L,w
		movwf		aargb2
		btfss		STATUS,C
		goto		$+6
		movf		POT_B1_H,w
		subwf		POT_B2_H,w
		movwf		aargb1
		decf		aargb1,f
		goto		$+4
		movf		POT_B1_H,w
		subwf		POT_B2_H,w
		movwf		aargb1

		clrf		aargb0
		movf		POT_VO_L,w
		btfsc		bit_etat_OV
		movf		POT_VF_L,w
		movwf		bargb1
		movf		POT_VO_H,w
		btfsc		bit_etat_OV
		movf		POT_VF_H,w
		movwf		bargb0
		call		fxd2416u		; Division de 24 bit  by 16 bit  to 24 bit.
									; Dividend = aargb0:aargb1:aargb2 (0 - most significant!) Divisor = bargb0:bargb1
		movf		aargb2,w
		movwf		cpt_B_L
		movf		aargb1,w
		movwf		cpt_B_H			; cpt_B_HL = 22  1282 pour 0,5  10,7 secondes.
		return

;***************************************************************************************************************
; Trois sous programme de lecture CAN et de mise  l'chelle dans la foule.
; En retour : temp1_H:temp1_L
; Variables temporaires : temp1
lecture_can1	; En retour : temp1_H:temp1_L = 0  1023
		movlw		.100
		movwf		temp1
		decfsz		temp1,f
		goto		$-1				; Petite tempo aprs la slection de l'entre CAN.
		bsf 		ADCON0,GO       ; Lance la conversion can.
		btfsc		ADCON0,GO		; Fini ?
		goto		$-1
		BANK1
		movf		ADRESL,w		; Lecture valeur du potentiomtre entre 0 et 1023.
		BANK0
		movwf		temp1_L
		movf		ADRESH,w
		movwf		temp1_H			;
		return						; En retour temp1_H:temp1_L = 0  1023
; ---------------------------
lecture_can2	; En retour : temp1_H:temp1_L = 0  511
		movlw		.100
		movwf		temp1
		decfsz		temp1,f
		goto		$-1				; Petite tempo aprs la slection de l'entre CAN.
		bsf 		ADCON0,GO       ; Lance la conversion can.
		btfsc		ADCON0,GO		; Fini ?
		goto		$-1
		BANK1
		movf		ADRESL,w		; Lecture valeur du potentiomtre entre 0 et 1023.
		BANK0
		movwf		temp1_L
		movf		ADRESH,w
		movwf		temp1_H			; temp1 = POT.
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			temp1_H,f
		rrf			temp1_L,f		; temp1 = POT/2.
		return						; En retour temp1_H:temp1_L = 0  511.
; ---------------------------
lecture_can3	; En retour : temp1_H:temp1_L = 0  255.
		movlw		.100
		movwf		temp1
		decfsz		temp1,f
		goto		$-1				; Petite tempo aprs la slection de l'entre CAN.
		bsf 		ADCON0,GO       ; Lance la conversion can.
		btfsc		ADCON0,GO		; Fini ?
		goto		$-1
		BANK1
		movf		ADRESL,w		; Lecture valeur du potentiomtre entre 0 et 1023.
		BANK0
		movwf		temp1_L
		movf		ADRESH,w
		movwf		temp1_H			; temp1 = POT.
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			temp1_H,f
		rrf			temp1_L,f		; temp1 = POT/2.
		bcf 		STATUS,C		; 0 -> CARRY (Avant dcalage de 1 bit vers la droite = division par 2).
		rrf			temp1_H,f
		rrf			temp1_L,f		; temp1 = POT/4.
		return						; En retour temp1_H:temp1_L = 0  255.

;***************************************************************************************************************
; Moyenne des 16 dernires mesures CAN 10 bits.
; pour viter une fluctuation incessante sur la lecture des potentiomtres.		
;
moy_can5:
		BANK2 
		movf		CAN5_L2+6,w
		movwf		CAN5_L2+7
		movf		CAN5_H2+6,w
		movwf		CAN5_H2+7
		movf		CAN5_L2+5,w
		movwf		CAN5_L2+6
		movf		CAN5_H2+5,w
		movwf		CAN5_H2+6
		movf		CAN5_L2+4,w
		movwf		CAN5_L2+5
		movf		CAN5_H2+4,w
		movwf		CAN5_H2+5
		movf		CAN5_L2+3,w
		movwf		CAN5_L2+4
		movf		CAN5_H2+3,w
		movwf		CAN5_H2+4
		movf		CAN5_L2+2,w
		movwf		CAN5_L2+3
		movf		CAN5_H2+2,w
		movwf		CAN5_H2+3
		movf		CAN5_L2+1,w
		movwf		CAN5_L2+2
		movf		CAN5_H2+1,w
		movwf		CAN5_H2+2
		movf		CAN5_L2+0,w
		movwf		CAN5_L2+1
		movf		CAN5_H2+0,w
		movwf		CAN5_H2+1
		BANK1
		movf		CAN5_L+7,w
		BANK2
		movwf		CAN5_L2+0
		BANK1
		movf		CAN5_H+7,w
		BANK2
		movwf		CAN5_H2+0
; ------------------
		BANK1
		movf		CAN5_L+6,w
		movwf		CAN5_L+7
		movf		CAN5_H+6,w
		movwf		CAN5_H+7
		movf		CAN5_L+5,w
		movwf		CAN5_L+6
		movf		CAN5_H+5,w
		movwf		CAN5_H+6
		movf		CAN5_L+4,w
		movwf		CAN5_L+5
		movf		CAN5_H+4,w
		movwf		CAN5_H+5
		movf		CAN5_L+3,w
		movwf		CAN5_L+4
		movf		CAN5_H+3,w
		movwf		CAN5_H+4
		movf		CAN5_L+2,w
		movwf		CAN5_L+3
		movf		CAN5_H+2,w
		movwf		CAN5_H+3
		movf		CAN5_L+1,w
		movwf		CAN5_L+2
		movf		CAN5_H+1,w
		movwf		CAN5_H+2
		movf		CAN5_L+0,w
		movwf		CAN5_L+1
		movf		CAN5_H+0,w
		movwf		CAN5_H+1
		movf		temp1_L,w
		movwf		CAN5_L+0
		movf		temp1_H,w
		movwf		CAN5_H+0		; Fin de dcalage des 16 dernires mesures.
; -----------------------------------------------------------------------------
		BANK2
		movf		CAN5_L2+7,w		; temp1_HL contient dj la valeur CAN5_HL_0.
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+7,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+6,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+5,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+4,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+3,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+2,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+1,w
		addwf		temp1_H,f		;
		movf		CAN5_L2+0,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H2+0,w
		addwf		temp1_H,f		;
;------------------------------------	
		BANK1
		movf		CAN5_L+7,w		; 
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+7,w
		addwf		temp1_H,f		;
		movf		CAN5_L+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+6,w
		addwf		temp1_H,f		;
		movf		CAN5_L+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+5,w
		addwf		temp1_H,f		;
		movf		CAN5_L+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+4,w
		addwf		temp1_H,f		;
		movf		CAN5_L+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+3,w
		addwf		temp1_H,f		;
		movf		CAN5_L+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+2,w
		addwf		temp1_H,f		;
		movf		CAN5_L+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN5_H+1,w		; Au dbut de l'addition, temp1_HL contient dj la valeur CAN5_HL_0.
		addwf		temp1_H,f		; Addition des 16 denires mesures.
; -----------------------------------------------------------------------------
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f		; Division par 16.
		BANK0
		return
		
moy_can6:
		BANK2 
		movf		CAN6_L2+6,w
		movwf		CAN6_L2+7
		movf		CAN6_H2+6,w
		movwf		CAN6_H2+7
		movf		CAN6_L2+5,w
		movwf		CAN6_L2+6
		movf		CAN6_H2+5,w
		movwf		CAN6_H2+6
		movf		CAN6_L2+4,w
		movwf		CAN6_L2+5
		movf		CAN6_H2+4,w
		movwf		CAN6_H2+5
		movf		CAN6_L2+3,w
		movwf		CAN6_L2+4
		movf		CAN6_H2+3,w
		movwf		CAN6_H2+4
		movf		CAN6_L2+2,w
		movwf		CAN6_L2+3
		movf		CAN6_H2+2,w
		movwf		CAN6_H2+3
		movf		CAN6_L2+1,w
		movwf		CAN6_L2+2
		movf		CAN6_H2+1,w
		movwf		CAN6_H2+2
		movf		CAN6_L2+0,w
		movwf		CAN6_L2+1
		movf		CAN6_H2+0,w
		movwf		CAN6_H2+1
		BANK1
		movf		CAN6_L+7,w
		BANK2
		movwf		CAN6_L2+0
		BANK1
		movf		CAN6_H+7,w
		BANK2
		movwf		CAN6_H2+0
; ------------------
		BANK1
		movf		CAN6_L+6,w
		movwf		CAN6_L+7
		movf		CAN6_H+6,w
		movwf		CAN6_H+7
		movf		CAN6_L+5,w
		movwf		CAN6_L+6
		movf		CAN6_H+5,w
		movwf		CAN6_H+6
		movf		CAN6_L+4,w
		movwf		CAN6_L+5
		movf		CAN6_H+4,w
		movwf		CAN6_H+5
		movf		CAN6_L+3,w
		movwf		CAN6_L+4
		movf		CAN6_H+3,w
		movwf		CAN6_H+4
		movf		CAN6_L+2,w
		movwf		CAN6_L+3
		movf		CAN6_H+2,w
		movwf		CAN6_H+3
		movf		CAN6_L+1,w
		movwf		CAN6_L+2
		movf		CAN6_H+1,w
		movwf		CAN6_H+2
		movf		CAN6_L+0,w
		movwf		CAN6_L+1
		movf		CAN6_H+0,w
		movwf		CAN6_H+1
		movf		temp1_L,w
		movwf		CAN6_L+0
		movf		temp1_H,w
		movwf		CAN6_H+0		; Fin de dcalage des 16 dernires mesures.
; -----------------------------------------------------------------------------
		BANK2
		movf		CAN6_L2+7,w		; temp1_HL contient dj la valeur CAN6_HL_0.
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+7,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+6,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+5,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+4,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+3,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+2,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+1,w
		addwf		temp1_H,f		;
		movf		CAN6_L2+0,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H2+0,w
		addwf		temp1_H,f		;
; -----------------------------------------
		BANK1
		movf		CAN6_L+7,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+7,w
		addwf		temp1_H,f		;
		movf		CAN6_L+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+6,w
		addwf		temp1_H,f		;
		movf		CAN6_L+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+5,w
		addwf		temp1_H,f		;
		movf		CAN6_L+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+4,w
		addwf		temp1_H,f		;
		movf		CAN6_L+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+3,w
		addwf		temp1_H,f		;
		movf		CAN6_L+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+2,w
		addwf		temp1_H,f		;
		movf		CAN6_L+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN6_H+1,w		; Au dbut de l'addition, temp1_HL contient dj la valeur CAN6_HL_0.
		addwf		temp1_H,f		; Addition des 16 denires mesures.
; -----------------
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f		; Division par 16.
		BANK0
		return

moy_can10:
		BANK2 
		movf		CAN10_L2+6,w
		movwf		CAN10_L2+7
		movf		CAN10_H2+6,w
		movwf		CAN10_H2+7
		movf		CAN10_L2+5,w
		movwf		CAN10_L2+6
		movf		CAN10_H2+5,w
		movwf		CAN10_H2+6
		movf		CAN10_L2+4,w
		movwf		CAN10_L2+5
		movf		CAN10_H2+4,w
		movwf		CAN10_H2+5
		movf		CAN10_L2+3,w
		movwf		CAN10_L2+4
		movf		CAN10_H2+3,w
		movwf		CAN10_H2+4
		movf		CAN10_L2+2,w
		movwf		CAN10_L2+3
		movf		CAN10_H2+2,w
		movwf		CAN10_H2+3
		movf		CAN10_L2+1,w
		movwf		CAN10_L2+2
		movf		CAN10_H2+1,w
		movwf		CAN10_H2+2
		movf		CAN10_L2+0,w
		movwf		CAN10_L2+1
		movf		CAN10_H2+0,w
		movwf		CAN10_H2+1
		BANK1
		movf		CAN10_L+7,w
		BANK2
		movwf		CAN10_L2+0
		BANK1
		movf		CAN10_H+7,w
		BANK2
		movwf		CAN10_H2+0
; ------------------
		BANK1
		movf		CAN10_L+6,w
		movwf		CAN10_L+7
		movf		CAN10_H+6,w
		movwf		CAN10_H+7
		movf		CAN10_L+5,w
		movwf		CAN10_L+6
		movf		CAN10_H+5,w
		movwf		CAN10_H+6
		movf		CAN10_L+4,w
		movwf		CAN10_L+5
		movf		CAN10_H+4,w
		movwf		CAN10_H+5
		movf		CAN10_L+3,w
		movwf		CAN10_L+4
		movf		CAN10_H+3,w
		movwf		CAN10_H+4
		movf		CAN10_L+2,w
		movwf		CAN10_L+3
		movf		CAN10_H+2,w
		movwf		CAN10_H+3
		movf		CAN10_L+1,w
		movwf		CAN10_L+2
		movf		CAN10_H+1,w
		movwf		CAN10_H+2
		movf		CAN10_L+0,w
		movwf		CAN10_L+1
		movf		CAN10_H+0,w
		movwf		CAN10_H+1
		movf		temp1_L,w
		movwf		CAN10_L+0
		movf		temp1_H,w
		movwf		CAN10_H+0		; Fin de dcalage des 16 dernires mesures.
; -----------------------------------------------------------------------------
		BANK2
		movf		CAN10_L2+7,w		; temp1_HL contient dj la valeur CAN10_HL_0.
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+7,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+6,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+5,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+4,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+3,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+2,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+1,w
		addwf		temp1_H,f		;
		movf		CAN10_L2+0,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H2+0,w
		addwf		temp1_H,f		;
; ---------------------------------------
		BANK1
		movf		CAN10_L+7,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+7,w
		addwf		temp1_H,f		;
		movf		CAN10_L+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+6,w
		addwf		temp1_H,f		;
		movf		CAN10_L+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+5,w
		addwf		temp1_H,f		;
		movf		CAN10_L+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+4,w
		addwf		temp1_H,f		;
		movf		CAN10_L+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+3,w
		addwf		temp1_H,f		;
		movf		CAN10_L+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+2,w
		addwf		temp1_H,f		;
		movf		CAN10_L+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN10_H+1,w		; Au dbut de l'addition, temp1_HL contient dj la valeur CAN10_HL_0.
		addwf		temp1_H,f		; Addition des 16 denires mesures.
; -----------------
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f		; Division par 16.
		BANK0
		return

moy_can11:
		BANK2 
		movf		CAN11_L2+6,w
		movwf		CAN11_L2+7
		movf		CAN11_H2+6,w
		movwf		CAN11_H2+7
		movf		CAN11_L2+5,w
		movwf		CAN11_L2+6
		movf		CAN11_H2+5,w
		movwf		CAN11_H2+6
		movf		CAN11_L2+4,w
		movwf		CAN11_L2+5
		movf		CAN11_H2+4,w
		movwf		CAN11_H2+5
		movf		CAN11_L2+3,w
		movwf		CAN11_L2+4
		movf		CAN11_H2+3,w
		movwf		CAN11_H2+4
		movf		CAN11_L2+2,w
		movwf		CAN11_L2+3
		movf		CAN11_H2+2,w
		movwf		CAN11_H2+3
		movf		CAN11_L2+1,w
		movwf		CAN11_L2+2
		movf		CAN11_H2+1,w
		movwf		CAN11_H2+2
		movf		CAN11_L2+0,w
		movwf		CAN11_L2+1
		movf		CAN11_H2+0,w
		movwf		CAN11_H2+1
		BANK1
		movf		CAN11_L+7,w
		BANK2
		movwf		CAN11_L2+0
		BANK1
		movf		CAN11_H+7,w
		BANK2
		movwf		CAN11_H2+0
; ------------------
		BANK1
		movf		CAN11_L+6,w
		movwf		CAN11_L+7
		movf		CAN11_H+6,w
		movwf		CAN11_H+7
		movf		CAN11_L+5,w
		movwf		CAN11_L+6
		movf		CAN11_H+5,w
		movwf		CAN11_H+6
		movf		CAN11_L+4,w
		movwf		CAN11_L+5
		movf		CAN11_H+4,w
		movwf		CAN11_H+5
		movf		CAN11_L+3,w
		movwf		CAN11_L+4
		movf		CAN11_H+3,w
		movwf		CAN11_H+4
		movf		CAN11_L+2,w
		movwf		CAN11_L+3
		movf		CAN11_H+2,w
		movwf		CAN11_H+3
		movf		CAN11_L+1,w
		movwf		CAN11_L+2
		movf		CAN11_H+1,w
		movwf		CAN11_H+2
		movf		CAN11_L+0,w
		movwf		CAN11_L+1
		movf		CAN11_H+0,w
		movwf		CAN11_H+1
		movf		temp1_L,w
		movwf		CAN11_L+0
		movf		temp1_H,w
		movwf		CAN11_H+0		; Fin de dcalage des 16 dernires mesures.
; -----------------------------------------------------------------------------
		BANK2
		movf		CAN11_L2+7,w		; temp1_HL contient dj la valeur CAN11_HL_0.
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+7,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+6,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+5,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+4,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+3,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+2,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+1,w
		addwf		temp1_H,f		;
		movf		CAN11_L2+0,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H2+0,w
		addwf		temp1_H,f		;
; ----------------------------
		BANK1
		movf		CAN11_L+7,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+7,w
		addwf		temp1_H,f		;
		movf		CAN11_L+6,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+6,w
		addwf		temp1_H,f		;
		movf		CAN11_L+5,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+5,w
		addwf		temp1_H,f		;
		movf		CAN11_L+4,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+4,w
		addwf		temp1_H,f		;
		movf		CAN11_L+3,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+3,w
		addwf		temp1_H,f		;
		movf		CAN11_L+2,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+2,w
		addwf		temp1_H,f		;
		movf		CAN11_L+1,w		;
		addwf		temp1_L,f
		btfsc		STATUS,C
		incf		temp1_H,f		;
		movf		CAN11_H+1,w		; Au dbut de l'addition, temp1_HL contient dj la valeur CAN11_HL_0.
		addwf		temp1_H,f		; Addition des 16 denires mesures.
; -----------------
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f
		bcf			STATUS,C
		rrf			temp1_H,f
		rrf			temp1_L,f		; Division par 16.
		BANK0
		return
	
;***************************************************************************************************************
; Unsigned division 24 bit / 16 bit -> 24 bit
;                   aargb0:aargb1:aargb2 / bargb0:bargb1 = > aargb0:aargb1:aargb2
; http://www.piclist.com/techref/microchip/math/div/24by16.htm
;
;Inputs:
;   Dividend - aargb0:aargb1:aargb2 (0 - most significant!)
;   Divisor  - bargb0:bargb1
;Temporary:
;   Counter  - loopcount
;   Remainder- remb0:remb1
;Output:
;   Quotient - aargb0:aargb1:aargb2
; Max timing: 4+24*(6+6+4+3+5)-1+3+2=584 cycles (with return)
; Min timing: 4+24*(6+6+5+5)-1+3+2=536   cycles (with return)
; 25-Sep-2000    Original version
; 20-Oct-2001    Made the loop one instruction shorter, comments review.
;   Nikolai Golovchenko shares this code:
;   Well, the routine can be made even simpler! This version is 5 instructionsshorter and 48 cycles faster.
;   Thanks to Zlatko Petkov for an idea ofremoving the extra shifts after the loop.
;   Nikolai Golovchenko. 5-Dec-2004.
fxd2416u:
		clrf	remb0
		clrf	remb1
		movlw	.24
		movwf	loopcount
loopu2416
		rlf		aargb2,w		; shift dividend left to move next bit to remainder
		rlf		aargb1,f		;
		rlf		aargb0,f		;
		rlf		remb1,f			; shift carry (next dividend bit) into remainder
		rlf		remb0,f
		rlf		aargb2,f		; finish shifting the dividend and save  carry in aargb2.0,
; since remainder can be 17 bit long in some cases
; (e.g. 0x800000/0xffff). this bit will also serve as the next result bit.
		movf	bargb1,w		; substract divisor from 16-bit remainder
		subwf	remb1,f			;
		movf	bargb0,w		;
		btfss	STATUS,C		;
		incfsz	bargb0,w		;
		subwf	remb0,f			;
; here we also need to take into account the 17th bit of remainder,which
; is in aargb2.0. if we don't have a borrow after subtracting from lower
; 16 bits of remainder, then there is no borrow regardless of 17th bit
; value. but, if we have the borrow, then that will depend on 17th bit
; value. if it is 1, then no final borrow will occur. if it is 0, borrow
; will occur. these values match the borrow flag polarity.
		btfsc	STATUS,C		; skpnc ; if no borrow after 16 bit subtraction
		bsf 	aargb2,0		; then there is no borrow in result. overwrite
; aargb2.0 with 1 to indicate no borrow.
; if borrow did occur, aargb2.0 already
; holds the final borrow value (0-borrow, 1-no borrow)
		btfsc	aargb2,0		; if no borrow after 17-bit subtraction
		goto	uok46ll			; skip remainder restoration.
		addwf	remb0,f			; restore higher byte of remainder. (w
; contains the value subtracted from it previously)
		movf	bargb1,w		; restore lower byte of remainder
		addwf	remb1,f			;
uok46ll
		decfsz	loopcount,f		; decrement counter
		goto	loopu2416		; and repeat the loop if not zero.
		return

;***************************************************************************************************************
; http://www.piclist.com/techref/microchip/math/mul/16x16umalin.htm
; malin@onspec.co.uk
; unsigned multiply of a2:a1 with b2:b1 leaving result in res4:res3:res2:res1
;	These 8 variables need to be defined
;
;	Program length	32 line
;	time 129 to 228 cycles
;	This program looks at the lsb of a1 to decide whether to add b1 to res2
;
;   and b2 to res3, with appropriate carrys
;	It then looks at the lsb of a2 to decide whether to add b1 to res3 and
;	b2 to res4, again with appropriate carrys.
;	The rotates then only have to be done 8 times
;
;	This is uses slightly more program but takes a little less time than
;	a routine that performs one 16 bit addition per rotate and 16 rotates
;
;	Multiple byte addition routine from Microchip AN617
;	Result registers used as loop counter from Bob Fehrenbach & Scott Dattalo
;
Multiply16x16bit
	clrf    res4
	clrf	res3
	clrf    res2
	movlw	0x80
	movwf	res1

nextbit
	rrf		a2,f
	rrf		a1,f
	btfss	STATUS,C
	goto	nobit_l
	movf	b1,w
	addwf	res2,f

	movf	b2, w
	btfsc	STATUS,C
	incfsz	b2, w
	addwf	res3, f
	btfsc	STATUS,C
	incf	res4, f
	bcf		STATUS,C

nobit_l
	btfss	a1, 7
	goto	nobit_h
	movf	b1,w
	addwf	res3,f
	movf	b2, w
	btfsc	STATUS,C
	incfsz	b2, w
	addwf	res4, f

nobit_h
	rrf		res4,f
	rrf		res3,f
	rrf		res2,f
	rrf		res1,f
	btfss	STATUS,C
	goto	nextbit
	return
	END